# bon -- bash on node

This is a bash script that helps run node.js or any other scripts.


## Why

Node.js is great for cli scripts.  Sometimes bash is an easier fit.
This script is great for running commands generated by another script / cli.
It pretends to be the script it delegates to.  It also changes directory
before running the target script, assuming its identity and source location.

The former helps with commands that need being run in a tty shell / context,
for example if the target cli generated an ssh connection / command string.
The latter, for any extra files the script may need, such as configuration.


## Use

Bon is most convenient when paired with node.js cli scripts. It makes use of
the `$NODE_MODULES` env var to `cd` into the target module source directory.
Being packaged as a module itself helps with making it a dependency.
I use `commander` and thus assume `--help` is available, appending to it in
a way that matches [commander](https://github.com/visionmedia/commander.js)'s
formatting style.


### Install

In order to run your commands from anywhere, install `bon` globally with
`npm install bon -g`.  If you want to make this automatic,
add the following to `package.json`:

```js
"scripts": {
  "preinstall": "npm i -g bon"
}
```

Of-course, your bon-enabled module should also be installed globally
so that its cli scripts can be found on the $PATH.

Bon depends on `coffee-script`.  If you rather not have that installed globally
as well, you'd have to add bon as a dependency so that `coffee` can be found.


### Convention

Naming the script command the same as its module name is expected as default.
Suppose your module is called `clier`, the entry in `package.json` should be
`"bin": { "clier" : "./node_modules/.bin/bon" }`, which looks for
`bin/clier.js`.  JavaScript is the default language, making the majority happy.
If CoffeeScript is preferred, just put the following code in it:

```js
#!/usr/bin/env node

require('coffee-script/register')
require('./clier.coffee')
```


### Configuration

There are config vars that override `bon`'s defaults.
If bon is exclusively paired with a single node script,
exporting them to the shell environment is fine,
certainly fine to first try it this way.

The easier option for serious work is to have the vars set with the aid of
`bin/bonvars.sh` - bon will source it, making available whatever is `export`ed.

To customize anything, including all the paths / file names, refer to your own
script in `package.json` - e.g. `"bin": { "clirest" : "./bin/clier.sh" }`, and
source bon with it.  Here is some example code:

```bash
#!/bin/bash

BON_NAME="clier" # must match module's name
BON_EXT="coffee" # $BON_SCRIPT would be "./bin/clier.coffee"
BON_SCRIPT="./bin/cli.coffee" #any path - ignoring BON_NAME and BON_EXT

source bon "$@" # provided bon is installed globally
```


## Features

### Path Check

By default bon checks if it changed directory to the right place.
It assumes there is a `package.json` with its `name` matching `$BON_NAME`.

Perhaps your script is not a node.js one.  Check any `$BON_CHECK_FILE`,
perhaps set to "README.md", and provide a `$BON_CHECK_GREP`
text or regex to match / verify with.

If you choose to trust where bon takes you to,
or else if your script is location-independent - then
set `BON_CHECK="no"`, and the path check will be skipped.


### Meta Commands

This was the reason bon was created to begin with.
Set `$BON_EVALIST` to a list of space-separated commands.
These are commands that generate commands to to be `eval`led.
The `$BON_EVALIST` perhaps could be shared among scripts via global `export`,
thus it would be possible to skip configuration in favor of convention.

The generated commands are verified to be *one line* long.
This is to prevent accidental errors that may cause damage.
A trailing `\n` is ok, even several trailing newlines are ok -
bash simply ignores it as a feature.

To develop commands with the target script, and skip the eval, run
`daps line <evalgen> ...` where `<evalgen>` is a meta-command that is
being developed and `...` are any optional args it may take.


## LICENSE

This is free and unencumbered public domain software.
For more information, see [UNLICENSE](http://unlicense.org).
